<?php
namespace Teknisa\EST\Factories;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Est53000ParametrosDeEstoqueFactory {
    #default service
    protected $connection;

    /* **************************************************
     * Construtor da Factory.
     *
     * @param EntityManager $entityManager
     */
    public function __construct(EntityManager $entityManager) {
        #initializes default service
        $this->connection = $entityManager->getConnection();
    }

    public function deleteAllDays($CDFILIAL, $NRORG) {
        #sets parameter data
        $params               = array();
        $params['P_CDFILIAL'] = $CDFILIAL;
        $params['P_NRORG']    = $NRORG;

        #defines function query
        $sql = "DELETE FROM DIAINVEFILI
                      WHERE CDFILIAL = :P_CDFILIAL
                        AND NRORG    = :P_NRORG";

        #prepares query and executes it with parameters
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function getDataForGeral($NRORG) {
        #sets parameter data
        $params               = array();
        $params['P_NRORG']    = $NRORG;

        #defines function query
        $sql = "SELECT CDPARESTOQ,
                       PE.NRORG,
                       VRPETOLEENTR,
                       IDTRATAVENC,
                       IDLANCDTMEN,
                       TRUNC(NRDIASENTR)          AS NRDIASENTR,
                       TRUNC(NRDIASSAID)          AS NRDIASSAID,
                       TRUNC(NRDIASAJUS)          AS NRDIASAJUS,
                       IDLANCDTMAI,
                       IDCONSFORNFJ,
                       IDBLOQFORIRR,
                       CDGROPERRETI,
                       CDGROPERSOLI,
                       TRUNC(NRMAXITROMA)         AS NRMAXITROMA,
                       CDGROPERFMES,
                       TRUNC(NRDIASENTRPD)        AS NRDIASENTRPD,
                       TRUNC(NRDIASSAIDPD)        AS NRDIASSAIDPD,
                       TRUNC(NRDIASAJUSPD)        AS NRDIASAJUSPD,
                       IDCONAUTIRRG,
                       IDBLOQPRESTO,
                       IDSUGCUSENTR,
                       IDALTDTFECDIA,
                       IDGTGTXTFDIA,
                       CDGROPERREQ,
                       IDCTRLESTNVPROD,
                       NRDIASMOVESTCEG,
                       CDGROPERLOT,
                       CDGROPERVISU,
                       NVL(IDOBFABRICLANCNF, 'N') AS IDOBFABRICLANCNF,
                       NVL(IDCONSMARCPRFAB, 'N')  AS IDCONSMARCPRFAB,
                       TRUNC(NRMESANTEMOVIEST) AS NRMESANTEMOVIEST,
                       DTMOVIESTINI,
                       DTMOVIESTFIN,
                       HRAGEMOVIEST,
                       IDPERMEXCITINVE,
                       PG.IDUTILDTPROC,
                       CDGROPERATUEST
                  FROM PARAMESTOQUE PE
                  LEFT JOIN PARAMGERAL PG
                  ON 1 = 1
                 WHERE PE.NRORG = :P_NRORG";

        #prepares query and executes it with parameters
        $std = $this->connection->prepare($sql);
        $std->execute($params);

        #fetch all result rows and returns
        return $std->fetchAll();
    }
    
    public function newParamestoque(){
        $sql = "INSERT INTO PARAMESTOQUE(CDPARESTOQ) (SELECT '1' FROM DUAL WHERE 0 = (SELECT COUNT(*) FROM PARAMESTOQUE))";

        #prepares query and executes it with parameters
        $std = $this->connection->prepare($sql);
        $std->execute();
    }

    public function getDataForProdutoField($NRORG) {
        #sets parameter data
        $params            = array();
        $params['P_NRORG'] = $NRORG;

        #defines function query
        $sql = "SELECT CDPRODUTO,
                       CDARVPROD,
                       NMPRODUTO,
                       CDNVPRODUTO
                  FROM PRODUTO
                 WHERE CDPRODUTO = CDPRODESTO
                   AND NRORG = :P_NRORG
              ORDER BY CDARVPROD";

        #prepares query and executes it with parameters
        $std = $this->connection->prepare($sql);
        $std->execute($params);

        #fetch all result rows and returns
        return $std->fetchAll();
    }
    
    public function validateStockControll( $idutilalmox, $idutillcestq, $idutillote, $cdfilial ) {
        #sets parameter data
        $params            = array();
        $params['P_CDFILIAL']        = $cdfilial;
        $operatorAlmox = $idutilalmox  == 'N'? '=' : '<>';
        $operatorLocal = $idutillcestq == 'N'? '=' : '<>';
        $operatorLote  = $idutillote   == 'N'? '=' : '<>';

        #defines function query
        $sql = "SELECT *
                FROM POSESTATU
                WHERE CDFILIAL = :P_CDFILIAL
                AND CDALMOXARIFE ".$operatorAlmox." ' '
                AND CDLOCALESTOQ ".$operatorLocal." ' '
                AND NRLOTEESTQ ".$operatorLote." ' '
                AND QTTOTALESTOQ <> 0";

        #prepares query and executes it with parameters
        $std = $this->connection->prepare($sql);
        $std->execute($params);

        #fetch all result rows and returns
        return $std->fetchAll();
    }

    public function getProductRequisitions($CDPRODUTO, $CDFILIAL, $NRORG) {
        #sets parameter data
        $params                = array();
        $params['P_CDPRODUTO'] = $CDPRODUTO;
        $params['P_CDFILIAL']  = $CDFILIAL;
        $params['P_NRORG']     = $NRORG;

        #defines function query
        $sql = "SELECT CDPRODUTO
                  FROM ROTFABRICA
                 WHERE CDPRODUTO = :P_CDPRODUTO
                   AND NRORG = :P_NRORG
                 UNION
                 SELECT CDPRODUTO
                  FROM ROTFABLOCAL
                 WHERE CDFILIAL = :P_CDFILIAL
                   AND CDPRODUTO = :P_CDPRODUTO
                   AND NRORG = :P_NRORG";

        #prepares query and executes it with parameters
        $std = $this->connection->prepare($sql);
        $std->execute($params);

        #fetch all result rows and returns
        return $std->fetchAll();
    }

    public function getDataForProdutoWidget($CDALMOXARIFE, $CDLOCALESTOQ, $CDFILIAL, $NRORG, $CDARVPROD = false, $IDCURVAABC = false) {
        #sets parameter data
        $params                   = array();
        $params['P_CDALMOXARIFE'] = $CDALMOXARIFE;
        $params['P_CDLOCALESTOQ'] = $CDLOCALESTOQ;
        $params['P_CDFILIAL']     = $CDFILIAL;
        $params['P_NRORG']        = $NRORG;

        if($CDARVPROD && $IDCURVAABC) {
            $params['P_CDARVPROD']  = $CDARVPROD;
            $params['P_IDCURVAABC'] = $IDCURVAABC;
        }

        #defines function query
        $sql = "SELECT CURV.IDCURVAABC,
                       CURV.CDPRODUTO,
                       CURV.IDMARCINVE,
                       PROD.NMPRODUTO,
                       CURV.CDFILIAL,
                       CURV.CDALMOXARIFE,
                       CURV.CDLOCALESTOQ,
                       PROD.CDARVPROD,
                       CURV.NRORG
                  FROM CURVAABC CURV
                  JOIN PRODUTO PROD
                    ON CURV.CDPRODUTO    = PROD.CDPRODUTO
                 WHERE ";
        if($CDARVPROD && $IDCURVAABC) {
            $sql .=   "PROD.CDARVPROD    = :P_CDARVPROD
                   AND CURV.IDCURVAABC   = :P_IDCURVAABC
                   AND ";
        }
        $sql .=       "CURV.CDALMOXARIFE = :P_CDALMOXARIFE
                   AND CURV.CDLOCALESTOQ = :P_CDLOCALESTOQ
                   AND CURV.CDFILIAL     = :P_CDFILIAL
                   AND CURV.NRORG        = :P_NRORG
              ORDER BY CURV.IDCURVAABC, PROD.CDARVPROD";

        #prepares query and executes it with parameters
        $std = $this->connection->prepare($sql);
        $std->execute($params);

        #fetch all result rows and returns
        return $std->fetchAll();
    }

    public function saveProduct($CDFILIAL, $CDALMOXARIFE, $CDLOCALESTOQ, $IDCURVAABC, $CDPRODUTO, $NRORG) {
        #sets parameter data
        $params               = array();
        $params['CDFILIAL'] = $CDFILIAL;
        $params['CDALMOXARIFE'] = $CDALMOXARIFE;
        $params['CDLOCALESTOQ'] = $CDLOCALESTOQ;
        $params['IDCURVAABC'] = $IDCURVAABC;
        $params['CDPRODUTO'] = $CDPRODUTO;
        $params['NRORG'] = $NRORG;

        $sql = "INSERT INTO CURVAABC( CDFILIAL,  CDALMOXARIFE,  CDLOCALESTOQ,  IDCURVAABC,  CDPRODUTO, IDMARCINVE,  NRORG, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, IDATIVO)
                     VALUES         (:CDFILIAL, :CDALMOXARIFE, :CDLOCALESTOQ, :IDCURVAABC, :CDPRODUTO,        'N', :NRORG,    SYSDATE,  SYSDATE,          null,           null,        null,         null,     'S')";

        #prepares query and executes it with parameters
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function updateProduct($CDFILIAL, $CDALMOXARIFE, $CDLOCALESTOQ, $IDCURVAABC, $CDPRODUTO, $NRORG, $IDCURVAABC_OLD) {
        #sets parameter data
        $params               = array();
        $params['CDFILIAL'] = $CDFILIAL;
        $params['CDALMOXARIFE'] = $CDALMOXARIFE;
        $params['CDLOCALESTOQ'] = $CDLOCALESTOQ;
        $params['IDCURVAABC'] = $IDCURVAABC;
        $params['CDPRODUTO'] = $CDPRODUTO;
        $params['NRORG'] = $NRORG;
        $params['IDCURVAABC_OLD'] = $IDCURVAABC_OLD;

        $sql = " UPDATE CURVAABC
                    SET IDCURVAABC = :IDCURVAABC
                   WHERE CDFILIAL = :CDFILIAL AND CDALMOXARIFE = :CDALMOXARIFE AND CDLOCALESTOQ = :CDLOCALESTOQ AND IDCURVAABC = :IDCURVAABC_OLD AND CDPRODUTO = :CDPRODUTO AND NRORG = :NRORG" ;



        #prepares query and executes it with parameters
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function saveGeral($row) {
        #sets parameter data
        $params                      = array();
        $params['VRPETOLEENTR']      = $row['VRPETOLEENTR'];
        $params['IDTRATAVENC']       = $row['IDTRATAVENC'];
        $params['IDLANCDTMEN']       = $row['IDLANCDTMEN'];
        $params['NRDIASENTR']        = $row['NRDIASENTR'];
        $params['NRDIASSAID']        = $row['NRDIASSAID'];
        $params['NRDIASAJUS']        = $row['NRDIASAJUS'];
        $params['IDLANCDTMAI']       = $row['IDLANCDTMAI'];
        $params['IDCONSFORNFJ']      = $row['IDCONSFORNFJ'];
        $params['IDBLOQFORIRR']      = $row['IDBLOQFORIRR'];
        $params['CDGROPERRETI']      = $row['CDGROPERRETI'];
        $params['CDGROPERSOLI']      = $row['CDGROPERSOLI'];
        $params['NRMAXITROMA']       = $row['NRMAXITROMA'];
        $params['CDGROPERFMES']      = $row['CDGROPERFMES'];
        $params['NRDIASENTRPD']      = $row['NRDIASENTRPD'];
        $params['NRDIASSAIDPD']      = $row['NRDIASSAIDPD'];
        $params['NRDIASAJUSPD']      = $row['NRDIASAJUSPD'];
        $params['IDCONAUTIRRG']      = $row['IDCONAUTIRRG'];
        $params['IDSUGCUSENTR']      = $row['IDSUGCUSENTR'];
        $params['IDBLOQPRESTO']      = $row['IDBLOQPRESTO'];
        $params['IDALTDTFECDIA']     = $row['IDALTDTFECDIA'];
        $params['IDGTGTXTFDIA']      = $row['IDGTGTXTFDIA'];
        $params['CDGROPERREQ']       = $row['CDGROPERREQ'];
        $params['NRDIASMOVESTCEG']   = $row['NRDIASMOVESTCEG'];
        $params['CDGROPERLOT']       = $row['CDGROPERLOT'];
        $params['CDGROPERVISU']      = $row['CDGROPERVISU'];
        $params['IDOBFABRICLANCNF']  = $row['IDOBFABRICLANCNF'];
        $params['IDCONSMARCPRFAB']   = $row['IDCONSMARCPRFAB'];
        $params['CDPARESTOQ']        = $row['CDPARESTOQ'];
        $params['NRORG']             = $row['NRORG'];
        $params['IDPERMEXCITINVE']   = $row['IDPERMEXCITINVE'];
        $params['NRMESANTEMOVIEST']  = $row['NRMESANTEMOVIEST'];
        $params['DTMOVIESTINI']      = isset($row['DTINTERVMOVIEST'])? $row['DTINTERVMOVIEST']['start'] : $row['DTMOVIESTINI'];
        $params['DTMOVIESTFIN']      = isset($row['DTINTERVMOVIEST'])? $row['DTINTERVMOVIEST']['end'] : $row['DTMOVIESTFIN'];
        $params['HRAGEMOVIEST']      = $row['HRAGEMOVIEST'];
        $params['CDGROPERATUEST']    = $row['CDGROPERATUEST'];
        
        $sql = "UPDATE PARAMESTOQUE
                    SET VRPETOLEENTR     = :VRPETOLEENTR,
                        IDTRATAVENC      = :IDTRATAVENC,
                        IDLANCDTMEN      = :IDLANCDTMEN,
                        NRDIASENTR       = :NRDIASENTR,
                        NRDIASSAID       = :NRDIASSAID,
                        NRDIASAJUS       = :NRDIASAJUS,
                        IDLANCDTMAI      = :IDLANCDTMAI,
                        IDCONSFORNFJ     = :IDCONSFORNFJ,
                        IDBLOQFORIRR     = :IDBLOQFORIRR,
                        CDGROPERRETI     = :CDGROPERRETI,
                        CDGROPERSOLI     = :CDGROPERSOLI,
                        NRMAXITROMA      = :NRMAXITROMA,
                        CDGROPERFMES     = :CDGROPERFMES,
                        NRDIASENTRPD     = :NRDIASENTRPD,
                        NRDIASSAIDPD     = :NRDIASSAIDPD,
                        NRDIASAJUSPD     = :NRDIASAJUSPD,
                        IDCONAUTIRRG     = :IDCONAUTIRRG,
                        IDSUGCUSENTR     = :IDSUGCUSENTR,
                        IDBLOQPRESTO     = :IDBLOQPRESTO,
                        IDALTDTFECDIA    = :IDALTDTFECDIA,
                        IDGTGTXTFDIA     = :IDGTGTXTFDIA,
                        CDGROPERREQ      = :CDGROPERREQ,
                        NRDIASMOVESTCEG  = :NRDIASMOVESTCEG,
                        CDGROPERLOT      = :CDGROPERLOT,
                        CDGROPERVISU     = :CDGROPERVISU,
                        IDOBFABRICLANCNF = :IDOBFABRICLANCNF,
                        IDCONSMARCPRFAB  = :IDCONSMARCPRFAB,
                        IDPERMEXCITINVE  = :IDPERMEXCITINVE,
                        NRMESANTEMOVIEST = :NRMESANTEMOVIEST,
                        DTMOVIESTINI     = :DTMOVIESTINI,
                        DTMOVIESTFIN     = :DTMOVIESTFIN,
                        HRAGEMOVIEST     = :HRAGEMOVIEST,
                        CDGROPERATUEST   = :CDGROPERATUEST
                    WHERE NRORG = :NRORG AND CDPARESTOQ = :CDPARESTOQ";
        #prepares query and executes it with parameters
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        #update tabela paramgeral
        $paramsGer                      = array();
        $paramsGer['IDUTILDTPROC']      = $row['IDUTILDTPROC'];
        $paramsGer['NRORG']             = $row['NRORG'];
        $sql = "UPDATE  PARAMGERAL
                        SET IDUTILDTPROC = :IDUTILDTPROC
                        WHERE NRORG = :NRORG";
        $stdGer = $this->connection->prepare($sql);
        $stdGer->execute($paramsGer);
        
        
    }
    
    public function callProcedureJobMovimentacao(){
        try {
            $procedure = new StoredProcedure($this->connection, 'AGENDAJOBMOVIEST');
            //$procedure->addParam(new Param('P_MENSAGEMERRO', Param::PARAM_OUTPUT, '', Param::PARAM_TYPE_STR, 5000));
            //$procedure->addParam(new Param('P_CDPRODUTO', Param::PARAM_INPUT, $cdproduto, Param::PARAM_TYPE_STR));
            return $procedure->execute();//['P_MENSAGEMERRO'];
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function saveUnidade($row) {

        #sets parameter data
        $params                      = array();
        $params['IDCTRLESTQ']        = $row['IDCTRLESTQ'];
        $params['IDESTONLINE']       = $row['IDESTONLINE'];
        $params['IDPERMESTNEG']      = $row['IDPERMESTNEG'];
        $params['DTPROCESSA']        = $row['DTPROCESSA'];
        $params['IDINVROTFMES']      = $row['IDINVROTFMES'];
        $params['VRPERCINVROT']      = isset($row['VRPERCINVROT']) ? $row['VRPERCINVROT'] : null;
        $params['IDORDIMPLANT']      = $row['IDORDIMPLANT'];
        $params['IDORDAJUINVE']      = $row['IDORDAJUINVE'];
        $params['IDBLQESTFMES']      = $row['IDBLQESTFMES'];
        $params['NRDIABLOQEST']      = $row['NRDIABLOQEST'];
        $params['IDINVROTDIA']       = $row['IDINVROTDIA'];
        $params['NRDIAFECHMES']      = $row['NRDIAFECHMES'];
        $params['CDALMOXARIFE']      = $row['CDALMOXARIFE'];
        $params['CDLOCALESTOQ']      = $row['CDLOCALESTOQ'];
        $params['IDUTCACALMOX']      = $row['IDUTCACALMOX'];
        $params['IDUTILALMOX']       = $row['IDUTILALMOX'];
        $params['IDUTILLCESTQ']      = $row['IDUTILLCESTQ'];
        $params['IDUTILLOTE']        = $row['IDUTILLOTE'];
        $params['DSNUMELOTE']        = $row['DSNUMELOTE'];
        $params['DSSEQULOTE']        = $row['DSSEQULOTE'];
        $params['IDAJUSIMPLAU']      = $row['IDAJUSIMPLAU'];
        $params['IDBLPRCOMEST']      = $row['IDBLPRCOMEST'];
        $params['IDBLOFECHMES']      = $row['IDBLOFECHMES'];
        $params['IDBLOQFECHNF']      = $row['IDBLOQFECHNF'];
        $params['IDGERARETAJU']      = $row['IDGERARETAJU'];
        $params['IDUTCODBARFI']      = $row['IDUTCODBARFI'];
        $params['IDBLOFERTNEV']      = $row['IDBLOFERTNEV'];
        $params['IDMENSESTNEG']      = $row['IDMENSESTNEG'];
        $params['IDBLFECHSNFE']      = $row['IDBLFECHSNFE'];
        $params['IDINVROTEST']       = $row['IDINVROTEST'];
        $params['CDTIPCUSTEST']      = $row['CDTIPCUSTEST'];
        $params['IDBLOQFECDIA']      = $row['IDBLOQFECDIA'];
        $params['DSORDENALOTE']      = $row['DSORDENALOTE'];
        $params['IDPERPROINAT']      = $row['IDPERPROINAT'];
        $params['IDINVEROTREQ']      = $row['IDINVEROTREQ'];
        $params['IDENTRSEQLOTE']     = $row['IDENTRSEQLOTE'];
        $params['IDENTRAUTLOTE']     = $row['IDENTRAUTLOTE'];
        $params['IDCONSDIAUTFEC']    = $row['IDCONSDIAUTFEC'];
        $params['IDFDIAENTREST']     = $row['IDFDIAENTREST'];
        $params['IDFDIASAIDEST']     = $row['IDFDIASAIDEST'];
        $params['IDFDIACARDAP']      = $row['IDFDIACARDAP'];
        $params['IDFDIARETIEST']     = $row['IDFDIARETIEST'];
        $params['IDFDIAREQEST']      = $row['IDFDIAREQEST'];
        $params['IDFDIAREQPLA']      = $row['IDFDIAREQPLA'];
        $params['IDFDIAEFEPLA']      = $row['IDFDIAEFEPLA'];
        $params['IDFDIAEFEFAT']      = $row['IDFDIAEFEFAT'];
        $params['VRPETOLEINVROT']    = $row['VRPETOLEINVROT'];
        $params['IDBLOFECHRETIEFET'] = $row['IDBLOFECHRETIEFET'];
        $params['IDREQCOMPEST']      = $row['IDREQCOMPEST'];
        $params['CDGRUPPRMOVEST']    = $row['CDGRUPPRMOVEST'];
        $params['CDGRUPPRCTRLTEM']   = $row['CDGRUPPRCTRLTEM'];
        $params['IDINVESTCEGO']      = $row['IDINVESTCEGO'];
        $params['CDSERVBAIESTCEG']   = $row['CDSERVBAIESTCEG'];
        $params['CDTPRETBXESTCEG']   = $row['CDTPRETBXESTCEG'];
        $params['IDLANCDTMENFIL']    = $row['IDLANCDTMENFIL'];
        $params['NRDIASENTRFIL']     = $row['NRDIASENTRFIL'];
        $params['NRDIASSAIDFIL']     = $row['NRDIASSAIDFIL'];
        $params['NRDIASAJUSFIL']     = $row['NRDIASAJUSFIL'];
        $params['IDLANCDTMAIFIL']    = $row['IDLANCDTMAIFIL'];
        $params['NRDIASENTRPDFIL']   = $row['NRDIASENTRPDFIL'];
        $params['NRDIASSAIDPDFIL']   = $row['NRDIASSAIDPDFIL'];
        $params['NRDIASAJUSPDFIL']   = $row['NRDIASAJUSPDFIL'];
        $params['IDPERMVISPOSEST']   = $row['IDPERMVISPOSEST'];
        $params['IDOBRIGAAJUINVE']   = $row['IDOBRIGAAJUINVE'];
        $params['IDCALCRECBRUTA']    = $row['IDCALCRECBRUTA'];
        $params['IDOBREALACEDIV']    = $row['IDOBREALACEDIV'];
        $params['IDBLOQFECINVPE']    = $row['IDBLOQFECINVPE'];
        $params['IDBLOQEFEPDIFEFEF'] = $row['IDBLOQEFEPDIFEFEF'];
        $params['IDBLOQAJUSNEVOL']   = $row['IDBLOQAJUSNEVOL'];
        $params['IDETIQPRODOL']      = $row['IDETIQPRODOL'];
        $params['IDENTRUNILOTE']     = $row['IDENTRUNILOTE'];
        $params['IDCTRLDISTRIB']     = $row['IDCTRLDISTRIB'];
        $params['DSPARFILFIS']       = $row['DSPARFILFIS'];
        $params['NRORG']             = $row['NRORG'];
        $params['CDFILIAL']          = $row['CDFILIAL'];
        $params['IDVISPOSAJINVE']    = $row['IDVISPOSAJINVE'];
        $params['IDNEXIBINFCPROD']   = $row['IDNEXIBINFCPROD'];
        $sql = " UPDATE PARAMFILIAL
                    SET IDCTRLESTQ        = :IDCTRLESTQ,
                        IDESTONLINE       = :IDESTONLINE,
                        IDPERMESTNEG      = :IDPERMESTNEG,
                        DTPROCESSA        = :DTPROCESSA,
                        IDINVROTFMES      = :IDINVROTFMES,
                        VRPERCINVROT      = :VRPERCINVROT,
                        IDORDIMPLANT      = :IDORDIMPLANT,
                        IDORDAJUINVE      = :IDORDAJUINVE,
                        IDBLQESTFMES      = :IDBLQESTFMES,
                        NRDIABLOQEST      = :NRDIABLOQEST,
                        IDINVROTDIA       = :IDINVROTDIA,
                        NRDIAFECHMES      = :NRDIAFECHMES,
                        CDALMOXARIFE      = :CDALMOXARIFE,
                        CDLOCALESTOQ      = :CDLOCALESTOQ,
                        IDUTCACALMOX      = :IDUTCACALMOX,
                        IDUTILALMOX       = :IDUTILALMOX,
                        IDUTILLCESTQ      = :IDUTILLCESTQ,
                        IDUTILLOTE        = :IDUTILLOTE,
                        DSNUMELOTE        = :DSNUMELOTE,
                        DSSEQULOTE        = :DSSEQULOTE,
                        IDAJUSIMPLAU      = :IDAJUSIMPLAU,
                        IDBLPRCOMEST      = :IDBLPRCOMEST,
                        IDBLOFECHMES      = :IDBLOFECHMES,
                        IDBLOQFECHNF      = :IDBLOQFECHNF,
                        IDGERARETAJU      = :IDGERARETAJU,
                        DSPARFILFIS       = :DSPARFILFIS,
                        IDUTCODBARFI      = :IDUTCODBARFI,
                        IDBLOFERTNEV      = :IDBLOFERTNEV,
                        IDMENSESTNEG      = :IDMENSESTNEG,
                        IDBLFECHSNFE      = :IDBLFECHSNFE,
                        IDINVROTEST       = :IDINVROTEST,
                        CDTIPCUSTEST      = :CDTIPCUSTEST,
                        IDBLOQFECDIA      = :IDBLOQFECDIA,
                        DSORDENALOTE      = :DSORDENALOTE,
                        IDPERPROINAT      = :IDPERPROINAT,
                        IDINVEROTREQ      = :IDINVEROTREQ,
                        IDENTRSEQLOTE     = :IDENTRSEQLOTE,
                        IDENTRAUTLOTE     = :IDENTRAUTLOTE,
                        IDCONSDIAUTFEC    = :IDCONSDIAUTFEC,
                        IDFDIAENTREST     = :IDFDIAENTREST,
                        IDFDIASAIDEST     = :IDFDIASAIDEST,
                        IDFDIACARDAP      = :IDFDIACARDAP,
                        IDFDIARETIEST     = :IDFDIARETIEST,
                        IDFDIAREQEST      = :IDFDIAREQEST,
                        IDFDIAREQPLA      = :IDFDIAREQPLA,
                        IDFDIAEFEPLA      = :IDFDIAEFEPLA,
                        IDFDIAEFEFAT      = :IDFDIAEFEFAT,
                        VRPETOLEINVROT    = :VRPETOLEINVROT,
                        IDBLOFECHRETIEFET = :IDBLOFECHRETIEFET,
                        IDREQCOMPEST      = :IDREQCOMPEST,
                        CDGRUPPRMOVEST    = :CDGRUPPRMOVEST,
                        CDGRUPPRCTRLTEM   = :CDGRUPPRCTRLTEM,
                        IDINVESTCEGO      = :IDINVESTCEGO,
                        CDSERVBAIESTCEG   = :CDSERVBAIESTCEG,
                        CDTPRETBXESTCEG   = :CDTPRETBXESTCEG,
                        IDLANCDTMENFIL    = :IDLANCDTMENFIL,
                        NRDIASENTRFIL     = :NRDIASENTRFIL,
                        NRDIASSAIDFIL     = :NRDIASSAIDFIL,
                        NRDIASAJUSFIL     = :NRDIASAJUSFIL,
                        IDLANCDTMAIFIL    = :IDLANCDTMAIFIL,
                        NRDIASENTRPDFIL   = :NRDIASENTRPDFIL,
                        NRDIASSAIDPDFIL   = :NRDIASSAIDPDFIL,
                        NRDIASAJUSPDFIL   = :NRDIASAJUSPDFIL,
                        IDPERMVISPOSEST   = :IDPERMVISPOSEST,
                        IDOBRIGAAJUINVE   = :IDOBRIGAAJUINVE,
                        IDCALCRECBRUTA    = :IDCALCRECBRUTA,
                        IDOBREALACEDIV    = :IDOBREALACEDIV,
                        IDBLOQFECINVPE    = :IDBLOQFECINVPE,
                        IDBLOQEFEPDIFEFEF = :IDBLOQEFEPDIFEFEF,
                        IDBLOQAJUSNEVOL   = :IDBLOQAJUSNEVOL,
                        IDETIQPRODOL      = :IDETIQPRODOL,
                        IDENTRUNILOTE     = :IDENTRUNILOTE,
                        IDCTRLDISTRIB     = :IDCTRLDISTRIB,
                        IDVISPOSAJINVE    = :IDVISPOSAJINVE,
                        IDNEXIBINFCPROD   = :IDNEXIBINFCPROD
                    WHERE NRORG = :NRORG AND CDFILIAL = :CDFILIAL";

        #prepares query and executes it with parameters
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function saveParametrosCurvaAbc($row) {
        #sets parameter data
        $params                      = array();
        $params['VRPECURVAA'] = $row['VRPECURVAA'];
        $params['VRPECURVAB'] = $row['VRPECURVAB'];
        $params['VRPECURVAC'] = $row['VRPECURVAC'];
        $params['NRORG']      = $row['NRORG'];
        $params['CDPARGERAL'] = $row['CDPARGERAL'];

        $sql = " UPDATE PARAMGERAL
                    SET VRPECURVAA = :VRPECURVAA,
                        VRPECURVAB = :VRPECURVAB,
                        VRPECURVAC = :VRPECURVAC
                    WHERE NRORG = :NRORG AND CDPARGERAL = :CDPARGERAL";

        #prepares query and executes it with parameters
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function getDataForUnidade($CDFILIAL, $NRORG) {
        #sets parameter data
        $params               = array();
        $params['P_CDFILIAL'] = $CDFILIAL;
        $params['P_NRORG']    = $NRORG;

        #defines function query
        $sql = "SELECT PARAM.NRORG,
                       PARAM.CDFILIAL,
                       PARAM.IDCTRLESTQ,
                       PARAM.IDNEXIBINFCPROD,
                       PARAM.IDESTONLINE,
                       PARAM.IDPERMESTNEG,
                       TPCST.NMTIPOCUSTO               AS NMTIPCUSTEST,
                       PARAM.DTPROCESSA,
                       PARAM.IDINVROTFMES,
                       PARAM.VRPERCINVROT,
                       PARAM.IDORDIMPLANT,
                       PARAM.IDORDAJUINVE,
                       PARAM.IDBLQESTFMES,
                       PARAM.NRDIABLOQEST,
                       PARAM.IDVISPOSAJINVE,
                       NVL(PARAM.IDINVROTDIA,'N')      AS IDINVROTDIA,
                       PARAM.NRDIAFECHMES,
                       PARAM.CDALMOXARIFE,
                       PARAM.CDLOCALESTOQ,
                       PARAM.IDUTCACALMOX,
                       ALMOX.DSALMOXARIFE,
                       LOCAL.DSLOCALESTOQ,
                       NVL(PARAM.IDUTILALMOX, 'N')     AS IDUTILALMOX,
                       NVL(PARAM.IDUTILLCESTQ, 'N')    AS IDUTILLCESTQ,
                       NVL(PARAM.IDUTILLOTE, 'N')      AS IDUTILLOTE,
                       PARAM.DSNUMELOTE,
                       PARAM.DSSEQULOTE,
                       PARAM.IDAJUSIMPLAU,
                       PARAM.IDBLPRCOMEST,
                       PARAM.IDBLOFECHMES,
                       PARAM.IDBLOQFECHNF,
                       PARAM.IDGERARETAJU,
                       PARAM.DSPARFILFIS,
                       SUBSTR(PARAM.DSPARFILFIS, 4, 1) AS IDBLOQESTNEG,
                       PARAM.IDUTCODBARFI,
                       PARAM.IDBLOFERTNEV,
                       PARAM.IDMENSESTNEG,
                       PARAM.IDBLFECHSNFE,
                       PARAM.IDINVROTEST,
                       PARAM.CDTIPCUSTEST,
                       PARAM.IDBLOQFECDIA,
                       PARAM.DSORDENALOTE,
                       PARAM.IDPERPROINAT,
                       PARAM.IDINVEROTREQ,
                       PARAM.IDENTRSEQLOTE,
                       PARAM.IDENTRAUTLOTE,
                       PARAM.IDCONSDIAUTFEC,
                       PARAM.IDFDIAENTREST,
                       PARAM.IDFDIASAIDEST,
                       PARAM.IDFDIACARDAP,
                       PARAM.IDFDIARETIEST,
                       PARAM.IDFDIAREQEST,
                       PARAM.IDFDIAREQPLA,
                       PARAM.IDFDIAEFEPLA,
                       PARAM.IDFDIAEFEFAT,
                       PARAM.VRPETOLEINVROT,
                       PARAM.IDBLOFECHRETIEFET,
                       PARAM.IDREQCOMPEST,
                       PARAM.CDGRUPPRMOVEST,
                       PARAM.CDGRUPPRCTRLTEM,
                       PARAM.IDINVESTCEGO,
                       PARAM.CDSERVBAIESTCEG,
                       PARAM.CDTPRETBXESTCEG,
                       PARAM.IDLANCDTMENFIL,
                       TRUNC(PARAM.NRDIASENTRFIL)      AS NRDIASENTRFIL,
                       TRUNC(PARAM.NRDIASSAIDFIL)      AS NRDIASSAIDFIL,
                       TRUNC(PARAM.NRDIASAJUSFIL)      AS NRDIASAJUSFIL,
                       PARAM.IDLANCDTMAIFIL,
                       TRUNC(PARAM.NRDIASENTRPDFIL)    AS NRDIASENTRPDFIL,
                       TRUNC(PARAM.NRDIASSAIDPDFIL)    AS NRDIASSAIDPDFIL,
                       TRUNC(PARAM.NRDIASAJUSPDFIL)    AS NRDIASAJUSPDFIL,
                       NVL(PARAM.IDUTMODPRODFIL, 'N')  AS IDUTMODPRODFIL,
                       NVL(PARAM.IDPERMVISPOSEST, 'S') AS IDPERMVISPOSEST,
                       PARAM.IDOBRIGAAJUINVE,
                       NVL(PARAM.IDCALCRECBRUTA, 'N')  AS IDCALCRECBRUTA,
                       NVL(PARAM.IDOBREALACEDIV, 'N')  AS IDOBREALACEDIV,
                       NVL(PARAM.IDBLOQFECINVPE, 'N')  AS IDBLOQFECINVPE,
                       PARAM.IDBLOQEFEPDIFEFEF,
                       PARAM.IDBLOQAJUSNEVOL,
                       NVL(PARAM.IDETIQPRODOL, 'N')    AS IDETIQPRODOL,
                       NVL(PARAM.IDENTRUNILOTE, 'N')   AS IDENTRUNILOTE,
                       NVL(PARAM.IDCTRLDISTRIB, 'N')   AS IDCTRLDISTRIB,
                       PARGE.CDPARGERAL,
                       PARGE.VRPECURVAA,
                       PARGE.VRPECURVAB,
                       PARGE.VRPECURVAC,
                       SERV.NMSERVICO,
                       TPRETI.NMTIPORETI
                  FROM PARAMFILIAL PARAM
             LEFT JOIN ALMOXARIFADO ALMOX
                    ON PARAM.CDFILIAL     = ALMOX.CDFILIAL
                   AND PARAM.CDALMOXARIFE = ALMOX.CDALMOXARIFE
             LEFT JOIN LOCALESTOQUE LOCAL
                    ON PARAM.CDFILIAL     = LOCAL.CDFILIAL
                   AND PARAM.CDALMOXARIFE = LOCAL.CDALMOXARIFE
                   AND PARAM.CDLOCALESTOQ = LOCAL.CDLOCALESTOQ
             LEFT JOIN TIPOCUSTO    TPCST
                    ON PARAM.CDTIPCUSTEST = TPCST.CDTIPOCUSTO
             LEFT JOIN PARAMGERAL   PARGE
                    ON 1 = 1
             LEFT JOIN SERVICO      SERV
                    ON PARAM.CDSERVBAIESTCEG = SERV.CDSERVFILI
                    AND :P_CDFILIAL = SERV.CDFILIAL
             LEFT JOIN TIPORETI     TPRETI
                    ON PARAM.CDTPRETBXESTCEG = TPRETI.CDTIPORETI
                 WHERE PARAM.CDFILIAL     = :P_CDFILIAL
                   AND PARAM.NRORG        = :P_NRORG";

        #prepares query and executes it with parameters
        $std = $this->connection->prepare($sql);
        $std->execute($params);

        #fetch all result rows and returns
        return $std->fetchAll();
    }

    public function deleteProdutosCurvaAbc($row) {
        $params                 = array();
        $params['CDFILIAL']     = $row['CDFILIAL'];
        $params['CDALMOXARIFE'] = $row['CDALMOXARIFE'];
        $params['CDLOCALESTOQ'] = $row['CDLOCALESTOQ'];
        $params['IDCURVAABC']   = $row['IDCURVAABC'];
        $params['CDPRODUTO']    = $row['CDPRODUTO'];
        $params['NRORG']        = $row['NRORG'];

        $sql = "DELETE FROM CURVAABC
                   WHERE CDFILIAL   = :CDFILIAL
                   AND CDALMOXARIFE = :CDALMOXARIFE
                   AND CDLOCALESTOQ = :CDLOCALESTOQ
                   AND CDPRODUTO    = :CDPRODUTO
                   AND IDCURVAABC   = :IDCURVAABC
                   AND NRORG        = :NRORG" ;
        #prepares query and executes it with parameters
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function insertAlmoxPadrao($filial){
        $params                 = array();
        $params['P_CDFILIAL']     = $filial;
        
        $sql = "INSERT INTO ALMOXARIFADO
                (CDFILIAL, CDALMOXARIFE, DSALMOXARIFE)
                (SELECT CDFILIAL, ' ', ' '
                 FROM PARAMFILIAL
                 WHERE CDFILIAL = :P_CDFILIAL
                 MINUS
                 SELECT CDFILIAL, CDALMOXARIFE, DSALMOXARIFE
                 FROM ALMOXARIFADO
                 WHERE CDFILIAL = :P_CDFILIAL)";
        #prepares query and executes it with parameters
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function insertLocalPadrao($filial){
        $params                 = array();
        $params['P_CDFILIAL']     = $filial;
        
        $sql = "INSERT INTO LOCALESTOQUE
                (CDFILIAL, CDALMOXARIFE, CDLOCALESTOQ, DSLOCALESTOQ)
                (SELECT CDFILIAL, CDALMOXARIFE, ' ', ' '
                 FROM ALMOXARIFADO
                 WHERE CDFILIAL = :P_CDFILIAL
                 MINUS
                 SELECT CDFILIAL, CDALMOXARIFE, CDLOCALESTOQ, DSLOCALESTOQ
                 FROM LOCALESTOQUE
                 WHERE CDFILIAL = :P_CDFILIAL)";
        #prepares query and executes it with parameters
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function getLotes($filial){
        $params               = array();
        $params['P_CDFILIAL']     = $filial;

        #defines function query
        $sql = "SELECT
                   CDFILIAL, NRLOTEESTQ, NRSUBLOTE
                FROM
                   LOTE
                WHERE
                   (CDFILIAL = :P_CDFILIAL) AND
                   (NRLOTEESTQ = ' ') AND
                   (NRSUBLOTE = ' ')";

        #prepares query and executes it with parameters
        $std = $this->connection->prepare($sql);
        $std->execute($params);

        #fetch all result rows and returns
        return $std->fetchAll();
    }
    
    public function insertLotesPadrao($filial){
        $params                 = array();
        $params['P_CDFILIAL']     = $filial;
        
        $sql = "INSERT INTO LOTE
                (CDFILIAL, CDPRODUTO, NRLOTEESTQ, NRSUBLOTE,
                IDSTATLOTE,
                DTENTRLOTE)
                (SELECT CDFILIAL, MAX(CDPRODUTO), ' ', ' ',
                        'L', SYSDATE
                 FROM PRODFILI
                 WHERE (CDFILIAL = :P_CDFILIAL)
                 GROUP BY CDFILIAL)";
        #prepares query and executes it with parameters
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function getParamplaneja($filial){
        $params               = array();
        $params['P_CDFILIAL']     = $filial;

        #defines function query
        $sql = "SELECT '1' FROM PARAMPLANEJA
                WHERE CDFILIAL = :P_CDFILIAL";

        #prepares query and executes it with parameters
        $std = $this->connection->prepare($sql);
        $std->execute($params);

        #fetch all result rows and returns
        return $std->fetchAll();
    }
    
    public function insertParamplaneja($filial){
        $params                 = array();
        $params['P_CDFILIAL']     = $filial;
        
        $sql = "INSERT INTO PARAMPLANEJA
                (CDFILIAL, CDPARPLANEJA)
                VALUES 
                (:P_CDFILIAL, '1')";
        #prepares query and executes it with parameters
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function clearExistingGroups() {
        $sql = "DELETE
                  FROM AUTGRUPOP
                 WHERE IDAUTGRUPOP  = 'GEST'
                    OR IDAUTGRUPOP  = 'GGER'
                    OR IDAUTGRUPOP  = 'GAUD'
                    AND NRAUTGRUPOP = '000'";

        $std = $this->connection->prepare($sql);
        $std->execute();
    }
    
    public function getDataForGroupFields() {
        #defines function query
        $sql = "SELECT MAX(CDGRUPGEST) CDGRUPGEST,
                       MAX(DSGRUPGEST) DSGRUPGEST ,
                       MAX(CDGRUPGGER) CDGRUPGGER,
                       MAX(DSGRUPGGER) DSGRUPGGER,
                       MAX(CDGRUPGAUD) CDGRUPGAUD,
                       MAX(DSGRUPGAUD) DSGRUPGAUD
                FROM(
                SELECT DECODE(IDAUTGRUPOP, 'GEST', A.CDGRUPOPER, '')CDGRUPGEST,
                       DECODE(IDAUTGRUPOP, 'GEST', DSGRUPOPER, '') DSGRUPGEST,
                       '' CDGRUPGGER,
                       '' DSGRUPGGER,
                       '' CDGRUPGAUD,
                       '' DSGRUPGAUD
                  FROM AUTGRUPOP A, GRUPOPER G
                 WHERE A.CDGRUPOPER = G.CDGRUPOPER
                   AND IDAUTGRUPOP = 'GEST'
                
                UNION ALL
                
                SELECT '' CDGRUPGEST,
                       '' DSGRUPGEST,
                       DECODE(IDAUTGRUPOP, 'GGER', A.CDGRUPOPER, '')CDGRUPGGER,
                       DECODE(IDAUTGRUPOP, 'GGER', DSGRUPOPER, '') DSGRUPGGER,
                       '' CDGRUPGAUD,
                       '' DSGRUPGAUD
                  FROM AUTGRUPOP A, GRUPOPER G
                 WHERE A.CDGRUPOPER = G.CDGRUPOPER
                   AND IDAUTGRUPOP = 'GGER'
                
                UNION ALL
                
                
                SELECT '' CDGRUPGEST,
                       '' DSGRUPGEST,
                       '' CDGRUPGGER,
                       '' DSGRUPGGER,
                       DECODE(IDAUTGRUPOP, 'GAUD', A.CDGRUPOPER, '')CDGRUPGAUD,
                       DECODE(IDAUTGRUPOP, 'GAUD', DSGRUPOPER, '') DSGRUPGAUD
                  FROM AUTGRUPOP A, GRUPOPER G
                 WHERE A.CDGRUPOPER = G.CDGRUPOPER
                   AND IDAUTGRUPOP = 'GAUD')";

        #prepares query and executes it with parameters
        $std = $this->connection->prepare($sql);
        $std->execute();

        #fetch all result rows and returns
        return $std->fetchAll();
    }

}