/*
TECNISA - TECNOLOGIA EM INFORMTICA

NOME: APURAC_PRODSEV

DESCRIO:
   PROCEDIMENTO PARA APURAO DE FATURAMENTO DE CLIENTES EM FILIAIS
   QUE UTILIZAM O TIPO DE CONTRATO PRODUTOS E SERVIOS.

CRIAO:
15/01/03 - ALESSANDRA CRISTINA

@C:\WIN_V2\SQL\END_CONV\ORACLE\6208_FAISTP_APUPRODSERV.SQL

Ex.:
-- EXECUTE APURAC_GESDIRETA('0003', '0000000002', '01/07/99', '10/07/99', 'M1', '0000000004');
-- RAISE_APPLICATION_ERROR(-20501, 'MENSAGEM', TRUE);
*/

CREATE OR REPLACE PROCEDURE APURAC_PRODSERV (
      P_CDFILIAL    IN FILIAL.CDFILIAL%TYPE,
      P_CDCLIENTE   IN CLIENTE.CDCLIENTE%TYPE,
      P_DATAINI     IN DATE,
      P_DATAFIM     IN DATE,
      P_NRAPURACFAT IN NOTAFISCAL.NRAPURACFAT%TYPE,
      P_IDGERANOTA  IN VARCHAR DEFAULT 'S') AS

   /* DECLARAO DO TIPO (RECORD) QUE CONTERA OS DADOS DO CURSOR */
   TYPE GDIRETARECTYPE IS RECORD (
      CDFILIAL       FILIAL.CDFILIAL%TYPE,
      CDCLIENTE      CLIENTE.CDCLIENTE%TYPE,
      DTFATURAM      DATE,
      CDSERVFILI     SERVICO.CDSERVFILI%TYPE,
      CDGRSERVFAT    GRUPSERVFAT.CDGRSERVFAT%TYPE,
      CDPRODUTO      PRODUTO.CDPRODUTO%TYPE,
      VRSERVICO      NUMBER,
      QTEFETIVOS     NUMBER);

   /* DECLARAO DO TIPO DO CURSOR */
   TYPE GDIRETACURTYPE IS REF CURSOR RETURN GDIRETARECTYPE;

   /* VARIVEIS QUE CONTERO O CURSOR E O REGISTRO */
   REC_GDIRETA    GDIRETARECTYPE;
   CUR_GDIRETA    GDIRETACURTYPE;

   V_CDFILIAPUR  CTRLAPURFAT.CDFILIAPUR%TYPE;

   /* VARIVEIS PARA CONTROLE DE PARAMETROS */
   STIDTIPNFEMP   VARCHAR2(01);
   STIDMINIEFET   VARCHAR2(01);
   STIDQTDEPREC   VARCHAR2(01);
   STIDUTEFEPLA   VARCHAR2(01);

   /* VARIVEIS PARA CONTROLE DE APURAO */
   QTDEMINIEFET   EFECLIFAT.QTCOMEFATU%TYPE;
   VRMINIEFET     EFECLIFAT.QTCOMEFATU%TYPE;
   V_CDSERVAJUS   SERVICO.CDSERVFILI%TYPE;
   V_QTEFETSERV   NUMBER;
   V_QTEFETGRUP   NUMBER;
   V_TTEFETSERV   NUMBER;
   V_VRSERVICO    NUMBER;
   V_VRSERVFATU   NUMBER;
   V_VRPERCCORR   NUMBER;
   V_EXISTE       NUMBER;

   /* VARIAVEIS PARA CONTROLE DE GRAVACAO DA NOTA FISCAL */
   V_CHAVECST     VARCHAR2(14);
   V_CSTTEMP      VARCHAR2(03);
   V_IDPRODSUBS   VARCHAR2(01);
   V_IDPRODANTE   VARCHAR2(01);

   V_CDSERIE      SERIENF.CDSERIE%TYPE;
   V_CDSERIEO     SERIENF.CDSERIE%TYPE;
   V_TTITENSNF    NUMBER;
   V_CONTITEM     NUMBER;
   V_NRLANCTONF   VARCHAR2(20);
   V_NRSEQITPROD  NUMBER;
   V_CDEMPRESA    FILIAL.CDEMPRESA%TYPE;
   V_CDINSCMUNI   FILIAL.CDINSCMUNI%TYPE;
   V_CDINSCESTA   FILIAL.CDINSCESTA%TYPE;
   V_CDEMPRESAO   FILIAL.CDEMPRESA%TYPE;
   V_CDINSCMUNIO  FILIAL.CDINSCMUNI%TYPE;
   V_CDINSCESTAO  FILIAL.CDINSCESTA%TYPE;
   V_IDTPIJURCLIE CLIENTE.IDTPIJURCLIE%TYPE;
   V_NRINSJURCLIE CLIENTE.NRINSJURCLIE%TYPE;
   V_IDNOVANOTA   VARCHAR2(01);
   V_VRNOTAFISC   NUMBER;
   V_PRXLANCESTQ  VARCHAR2(10);
   V_VRCREDIDEBI  NOTAFISCAL.VRNOTAFISC%TYPE;

   V_DSAJUCTREC  AJCTRECEBER.DSAJUSTEREC%TYPE;
   V_ENCARGO     ENCARGO.CDENCARGO%TYPE;

   V_CDGRUPOFISC  VARCHAR2(03);
   V_DESCDOTACAO  VARCHAR2(01);
   V_CDAJUSTPADR  VARCHAR2(02);
   V_CDENCREDITO  VARCHAR2(02);
   V_CDENCDEBITO  VARCHAR2(02);

   V_PERCGERNFSERV FLOAT;
   V_VRTOTALFATUR  FLOAT;
   V_VRTOTNFSERV   FLOAT;

   V_EMPREATAC     FILIAL.CDEMPRESA%TYPE;
   V_EMPREOPER     EMPRESA.CDEMPRESA%TYPE;
   V_CDPRODUTO     PRODUTO.CDPRODUTO%TYPE;
   V_NMPRODUTO     PRODUTO.NMPRODUTO%TYPE;
   V_CDARVPROD     PRODUTO.CDARVPROD%TYPE;
   V_QTITEMPROD    FLOAT;
   V_VRUNITPRFATU  FLOAT;

   /* PROCEDIMENTO QUE ABRE O CURSOS DE ACORDO COM ALGUNS PARAMETROS
    PARAMETROS:
    1. CURSOR A SER ABERTO
    2. INDICADOR DO CONTROLE DE MINIMO DE EFETIVOS (POR DIA OU POR PERIODO)
    3. INDICADOR DA UTILIZACAO DE EFETIVOS DO PLANEJAMENTO
    4. INDICADOR DA UTILIZACAO DE GRUPOS DE SERVICOS */
   PROCEDURE OPENGDIRETACURSOR (C_GDIRETA  IN OUT GDIRETACURTYPE,
                                V_MINEFET  IN VARCHAR2,
                                V_UTPLANEJ IN VARCHAR2,
                                V_GRUPO    IN VARCHAR2) IS
   BEGIN
      IF (V_MINEFET = '2') THEN
      /* CONTROLA POR PERIODO */
         OPEN C_GDIRETA FOR
            SELECT CDFILIAL, CDCLIENTE, NULL DTFAT, CDSERVFILI, CDGRSERVFAT,
                   CDPRODUTO, VRSERVICO, SUM(QTEFETIVOS)
            FROM SUM_EFETIVOS
            WHERE DTFATURAM BETWEEN P_DATAINI AND P_DATAFIM AND
                  ((IDFATUPLAN = 'F') OR (V_UTPLANEJ = 'S')) AND
                  CDFILIAL   = V_CDFILIAPUR AND
                  CDCLIENTE  = P_CDCLIENTE
            GROUP BY CDFILIAL, CDCLIENTE, CDSERVFILI, CDGRSERVFAT, CDPRODUTO,
                     VRSERVICO
            HAVING (SUM(QTEFETIVOS) <> 0) AND
                   (((CDGRSERVFAT IS NULL) AND (V_GRUPO = 'N')) OR
                    ((CDGRSERVFAT IS NOT NULL) AND (V_GRUPO = 'S')));
      END IF;

      IF (V_MINEFET = '1') THEN
      /* CONTROLA POR DIA */
         OPEN C_GDIRETA FOR
            SELECT CDFILIAL, CDCLIENTE, DTFATURAM, CDSERVFILI, CDGRSERVFAT,
                   CDPRODUTO, VRSERVICO, SUM(QTEFETIVOS)
            FROM SUM_EFETIVOS
            WHERE DTFATURAM BETWEEN P_DATAINI AND P_DATAFIM AND
                  ((IDFATUPLAN = 'F') OR (V_UTPLANEJ = 'S')) AND
                  CDFILIAL   = V_CDFILIAPUR AND
                  CDCLIENTE  = P_CDCLIENTE
            GROUP BY CDFILIAL, CDCLIENTE, DTFATURAM, CDSERVFILI, CDGRSERVFAT,
                     CDPRODUTO, VRSERVICO
            HAVING (SUM(QTEFETIVOS) <> 0) AND
                   (((CDGRSERVFAT IS NULL) AND (V_GRUPO = 'N')) OR
                    ((CDGRSERVFAT IS NOT NULL) AND (V_GRUPO = 'S')));
      END IF;
   END OPENGDIRETACURSOR;

BEGIN

   V_VRCREDIDEBI  := 0;
   V_VRTOTALFATUR := 0;
   V_VRTOTNFSERV  := 0;

   V_CHAVECST := P_CDFILIAL || P_NRAPURACFAT;

   /* FILIAL QUE EST SENDO APURADA (P_FILIAL = FILIPROC)*/
   SELECT MAX(CDFILIAPUR)
   INTO V_CDFILIAPUR
   FROM CTRLAPURFAT
   WHERE CDFILIAL    = P_CDFILIAL AND
         NRAPURACFAT = P_NRAPURACFAT;

   /* BUSCA AS INFORMAOES PARA CADASTRAMENTO DE AJUSTE */

   SELECT MAX(SUBSTR(DSAJUCTREC,1,20))
   INTO V_DSAJUCTREC
   FROM TIPOAJRECEBER
   WHERE V_CDAJUSTPADR = CDAJUCTREC;


   /* APAGA OS REGISTROS DA TRAB_APURAC_GD DE APURACOES ANTERIORES */
   DELETE FROM TRAB_APURAC_GD
   WHERE CDFILIALAPGD  = P_CDFILIAL    AND
         NRAPURACGD    = P_NRAPURACFAT AND
         CDCLIENTEAPGD = P_CDCLIENTE;

   /* BUSCA OS PARAMETROS DA PARAMGERAL E PARAMFILIAL */
   SELECT MAX(SUBSTR(DSPARAMFATGE,11,1)) INTO STIDTIPNFEMP
   FROM PARAMGERAL;

   IF (STIDTIPNFEMP IS NULL) OR (RTRIM(STIDTIPNFEMP) = '') THEN
      STIDTIPNFEMP := '2';
      -- REFEIO
   END IF;

   SELECT MAX(SUBSTR(DSPARAMFATFI,01,1)),
          MAX(SUBSTR(DSPARAMFATFI,02,1)),
          MAX(SUBSTR(DSPARAMFATFI,15,1)),
          MAX(SUBSTR(DSPARAMFATFI,56,1)),          /*DESCONTO DE DOTAO*/
          MAX(SUBSTR(DSPARAMFATFI,57,2)),          /*AJUSTE PADRAO*/
          MAX(SUBSTR(DSPARAMFATFI,59,2)),          /*ENCARGO DEBITO*/
          MAX(SUBSTR(DSPARAMFATFI,61,2)),          /*ENCARGO CREDITO*/
          MAX(SUBSTR(DSPARAMFATFI,31,6)),          /*PERCENTUAL FATURAMENTO*/
          MAX(SUBSTR(DSPARAMFATFI,39,2)),          /*Empresa atacadista*/
          MAX(SUBSTR(DSPARAMFATFI,41,2)),          /*Empresa operacional*/
          RTRIM(MAX(SUBSTR(DSPARAMFATGE,12,10)))   /*Prod.Faturamento(Prestao de Servios)*/

   INTO STIDMINIEFET,  STIDQTDEPREC,    STIDUTEFEPLA,
        V_DESCDOTACAO, V_CDAJUSTPADR,   V_CDENCREDITO,
        V_CDENCDEBITO, V_PERCGERNFSERV, V_EMPREATAC,
        V_EMPREOPER,   V_CDPRODUTO
   FROM PARAMFILIAL PF, PARAMGERAL
   WHERE PF.CDFILIAL = V_CDFILIAPUR;

   V_PERCGERNFSERV  := V_PERCGERNFSERV/1000;  /* RECUPERA CASAS DECIMAIS*/

      /* BUSCA AS INFORMAOES PARA CADASTRAMENTO DA NOTA FISCAL */
   SELECT MAX(CDSERIE), MAX(CDINSCESTA), MAX(CDINSCMUNI)
   INTO V_CDSERIE, V_CDINSCESTA, V_CDINSCMUNI
   FROM FILIAL
   WHERE CDFILIAL  = V_CDFILIAPUR   AND
         CDEMPRESA = V_EMPREATAC;

   SELECT MAX(CDINSCMUNI), MAX(CDINSCESTA), MAX(CDSERIE)
   INTO V_CDINSCMUNIO, V_CDINSCESTAO, V_CDSERIEO
   FROM EMPRCNTRFIL
   WHERE CDEMPRESA = V_EMPREOPER AND
         CDFILIAL  = V_CDFILIAPUR;

   IF (STIDMINIEFET IS NULL) OR (STIDMINIEFET = ' ') THEN
      STIDMINIEFET := '1';
      -- POR DIA
   END IF;
   IF (STIDQTDEPREC IS NULL) OR (STIDQTDEPREC = ' ') THEN
      STIDQTDEPREC := '1';
      -- POR QUANTIDADE
   END IF;
   IF (STIDUTEFEPLA IS NULL) OR (STIDUTEFEPLA = ' ') THEN
      STIDUTEFEPLA := 'N';
      -- NAO UTILIZA EFETIVOS DO PLANEJAMENTO
   END IF;

   SELECT SUM(VRNOTAFISC)
   INTO V_VRCREDIDEBI
   FROM NOTAFISCAL N
   WHERE (N.CDFILIAL     = V_CDFILIAPUR)                  AND
         (N.CDTIPOOPER   = '12')                          AND
         (N.IDENTRSAIDOP = 'S')                           AND
         (N.DTENTRSAID BETWEEN P_DATAINI AND P_DATAFIM);

   IF V_VRCREDIDEBI < 0 THEN
      V_VRCREDIDEBI := V_VRCREDIDEBI * (-1);
      V_ENCARGO    := V_CDENCDEBITO;
   ELSIF V_VRCREDIDEBI > 0 THEN
      V_ENCARGO    :=  V_CDENCREDITO;
   END IF;


   CONSULTA_GRUPOFISCAL('C', P_CDCLIENTE, V_CDFILIAPUR, 'S', '01', V_CDGRUPOFISC);

   /* ABRE O CURSOR */
   OPENGDIRETACURSOR(CUR_GDIRETA, STIDMINIEFET, STIDUTEFEPLA, 'N');
   LOOP
      FETCH CUR_GDIRETA INTO REC_GDIRETA;
      EXIT WHEN CUR_GDIRETA%NOTFOUND;

      /* CALCULA O MINIMO DE EFETIVOS */
      QTDEMINIEFET := UTIL_APURAC.CALCEFETMINI(REC_GDIRETA.CDFILIAL,
                                               REC_GDIRETA.CDCLIENTE,
                                               REC_GDIRETA.CDSERVFILI,
                                               REC_GDIRETA.CDGRSERVFAT,
                                               REC_GDIRETA.DTFATURAM);

      V_VRSERVICO  := REC_GDIRETA.VRSERVICO;
      V_QTEFETSERV := REC_GDIRETA.QTEFETIVOS;

      /* EFETUA O AJUSTE DAS QUANTIDADES OU VALORES DOS SERVICOS */
      IF (STIDMINIEFET = '1') THEN
      -- POR DIA
         IF (STIDQTDEPREC = '1') THEN
         -- POR QUANTIDADE
            IF (REC_GDIRETA.QTEFETIVOS < QTDEMINIEFET) THEN
               V_QTEFETSERV := QTDEMINIEFET;
            END IF;
         ELSE
         -- POR PRECO
            UTIL_APURAC.PRECOCORR(REC_GDIRETA.CDFILIAL,
                                  REC_GDIRETA.CDCLIENTE,
                                  REC_GDIRETA.CDSERVFILI,
                                  REC_GDIRETA.CDGRSERVFAT,
                                  V_QTEFETSERV,
                                  QTDEMINIEFET,
                                  'S',
                                  V_VRSERVICO,
                                  V_VRPERCCORR);
         END IF;
      ELSE
      -- POR PERIODO

         /* VERIFICA SE JA FOI GRAVADO REGISTRO PARA AJUSTE DE EFETIVOS
            POR QUANTIDADE */
         SELECT COUNT(*)
         INTO V_EXISTE
         FROM TRAB_APURAC_GD
         WHERE CDFILIALAPGD   = P_CDFILIAL AND
               NRAPURACGD     = P_NRAPURACFAT AND
               CDCLIENTEAPGD  = REC_GDIRETA.CDCLIENTE AND
               CDSERVFILIAPGD = REC_GDIRETA.CDSERVFILI AND
               IDAJUSEFETAPGD = 'S';

         IF (V_EXISTE = 0) THEN
            /* CALCULA O TOTAL DE EFETIVOS DO PERIODO */
            SELECT SUM(QTEFETIVOS)
            INTO V_TTEFETSERV
            FROM SUM_EFETIVOS
            WHERE (DTFATURAM BETWEEN P_DATAINI AND P_DATAFIM) AND
                  ((IDFATUPLAN = 'F') OR (STIDUTEFEPLA = 'S')) AND
                  CDFILIAL   = REC_GDIRETA.CDFILIAL AND
                  CDCLIENTE  = REC_GDIRETA.CDCLIENTE AND
                  CDSERVFILI = REC_GDIRETA.CDSERVFILI;

            IF (STIDQTDEPREC = '1') THEN
            -- POR QUANTIDADE
               IF (V_TTEFETSERV < QTDEMINIEFET) THEN
                  SELECT MAX(VRSERVFATU)
                  INTO V_VRSERVFATU
                  FROM SERVCLIE
                  WHERE CDFILIAL   = REC_GDIRETA.CDFILIAL AND
                        CDCLIENTE  = REC_GDIRETA.CDCLIENTE AND
                        CDSERVFILI = REC_GDIRETA.CDSERVFILI;

                  IF (V_VRSERVFATU IS NULL) THEN
                     V_VRSERVFATU := 0;
                  END IF;

                  CONSULTA_CST('S', V_CDEMPRESA, REC_GDIRETA.CDPRODUTO, V_CDINSCESTA,
                               '01', P_CDFILIAL, P_NRAPURACFAT, V_CDGRUPOFISC);

                  SELECT NVL(MAX(CDSITTRIBTMP),'00')
                  INTO V_CSTTEMP
                  FROM CSTTEMP
                  WHERE NRCSTTEMP = V_CHAVECST;

                  IF SUBSTR(LPAD(V_CSTTEMP,2,'0'),2,1) = '6' THEN
                     V_IDPRODSUBS := 'S';
                  ELSE
                     V_IDPRODSUBS := 'N';
                  END IF;

                  /* GRAVA O AJUSTE DA QUANTIDADE DE EFETIVOS */
                  INSERT INTO TRAB_APURAC_GD
                  (CDFILIALAPGD, NRAPURACGD, CDCLIENTEAPGD,
                   CDSERVFILIAPGD, DTAPURACFATGD, CDPRODUTOAPGD,
                   NREFETIVOSAPGD, VRPRECUNITAPGD,
                   IDAJUSEFETAPGD)
                  VALUES (P_CDFILIAL, P_NRAPURACFAT, REC_GDIRETA.CDCLIENTE,
                          REC_GDIRETA.CDSERVFILI, P_DATAINI, REC_GDIRETA.CDPRODUTO,
                          QTDEMINIEFET - V_TTEFETSERV, V_VRSERVFATU, 'S');

                  DELETE CSTTEMP
                  WHERE NRCSTTEMP = V_CHAVECST;
               END IF;
            ELSE
            -- POR PRECO
               IF (V_TTEFETSERV < QTDEMINIEFET) THEN
                  UTIL_APURAC.PRECOCORR(REC_GDIRETA.CDFILIAL,
                                        REC_GDIRETA.CDCLIENTE,
                                        REC_GDIRETA.CDSERVFILI,
                                        REC_GDIRETA.CDGRSERVFAT,
                                        V_TTEFETSERV,
                                        QTDEMINIEFET,
                                        'S',
                                        V_VRSERVICO,
                                        V_VRPERCCORR);
               END IF;
            END IF;
         END IF;
      END IF;

      /* GRAVA OS SERVICOS NA TRAB_APURAC_GD JA COM AS QUANTIDADES
         E VALORES CORRIGIDOS */
      SELECT COUNT(*)
      INTO V_EXISTE
      FROM TRAB_APURAC_GD
      WHERE CDFILIALAPGD   = P_CDFILIAL AND
            NRAPURACGD     = P_NRAPURACFAT AND
            CDCLIENTEAPGD  = REC_GDIRETA.CDCLIENTE AND
            CDSERVFILIAPGD = REC_GDIRETA.CDSERVFILI AND
            VRPRECUNITAPGD = ROUND(V_VRSERVICO,2);

      IF (V_EXISTE = 0) THEN
         CONSULTA_CST('S', V_CDEMPRESA, REC_GDIRETA.CDPRODUTO, V_CDINSCESTA,
                      '01', P_CDFILIAL, P_NRAPURACFAT, V_CDGRUPOFISC);

         SELECT NVL(MAX(CDSITTRIBTMP),'00')
         INTO V_CSTTEMP
         FROM CSTTEMP
         WHERE NRCSTTEMP = V_CHAVECST;

         IF SUBSTR(LPAD(V_CSTTEMP,2,'0'),2,1) = '6' THEN
            V_IDPRODSUBS := 'S';
         ELSE
            V_IDPRODSUBS := 'N';
         END IF;

         INSERT INTO TRAB_APURAC_GD
         (CDFILIALAPGD, NRAPURACGD, CDCLIENTEAPGD,
          CDSERVFILIAPGD, DTAPURACFATGD, CDPRODUTOAPGD,
          NREFETIVOSAPGD, VRPRECUNITAPGD,
          IDAJUSEFETAPGD)
         VALUES (P_CDFILIAL, P_NRAPURACFAT, REC_GDIRETA.CDCLIENTE,
                 REC_GDIRETA.CDSERVFILI, P_DATAINI, REC_GDIRETA.CDPRODUTO,
                 V_QTEFETSERV, ROUND(V_VRSERVICO,2), 'N');

         DELETE CSTTEMP
         WHERE NRCSTTEMP = V_CHAVECST;
      ELSE
         UPDATE TRAB_APURAC_GD
         SET NREFETIVOSAPGD = NREFETIVOSAPGD + V_QTEFETSERV
         WHERE CDFILIALAPGD   = P_CDFILIAL AND
               NRAPURACGD     = P_NRAPURACFAT AND
               CDCLIENTEAPGD  = REC_GDIRETA.CDCLIENTE AND
               CDSERVFILIAPGD = REC_GDIRETA.CDSERVFILI AND
               VRPRECUNITAPGD = ROUND(V_VRSERVICO,2);
      END IF;
   END LOOP;
   CLOSE CUR_GDIRETA;

   /* ABRE O CURSOR PARA TODOS OS SERVICOS QUE POSSUEM GRUPOS */
   OPENGDIRETACURSOR(CUR_GDIRETA, STIDMINIEFET, STIDUTEFEPLA, 'S');
   LOOP
      FETCH CUR_GDIRETA INTO REC_GDIRETA;
      EXIT WHEN CUR_GDIRETA%NOTFOUND;

      /* CALCULA O MINIMO DE EFETIVOS */
      QTDEMINIEFET := UTIL_APURAC.CALCEFETMINI(REC_GDIRETA.CDFILIAL,
                                               REC_GDIRETA.CDCLIENTE,
                                               REC_GDIRETA.CDSERVFILI,
                                               REC_GDIRETA.CDGRSERVFAT,
                                               REC_GDIRETA.DTFATURAM);

      V_VRSERVICO  := REC_GDIRETA.VRSERVICO;
      V_QTEFETSERV := REC_GDIRETA.QTEFETIVOS;

      SELECT SUM(QTEFETIVOS)
      INTO V_QTEFETGRUP
      FROM SUM_EFETIVOS
      WHERE (DTFATURAM = REC_GDIRETA.DTFATURAM) AND
            ((IDFATUPLAN = 'F') OR (STIDUTEFEPLA = 'S')) AND
            CDFILIAL    = REC_GDIRETA.CDFILIAL AND
            CDCLIENTE   = REC_GDIRETA.CDCLIENTE AND
            CDGRSERVFAT = REC_GDIRETA.CDGRSERVFAT;

      SELECT MAX(CDSERAJEFEGSF)
      INTO V_CDSERVAJUS
      FROM GRUPSERVFAT
      WHERE CDFILIAL    = REC_GDIRETA.CDFILIAL AND
            CDCLIENTE   = REC_GDIRETA.CDCLIENTE AND
            CDGRSERVFAT = REC_GDIRETA.CDGRSERVFAT;

      /* EFETUA O AJUSTE DAS QUANTIDADES OU VALORES DOS SERVICOS */
      IF (STIDMINIEFET = '1') THEN
      -- POR DIA
         IF (STIDQTDEPREC = '1') THEN
         -- POR QUANTIDADE
            IF (V_QTEFETGRUP < QTDEMINIEFET) AND (REC_GDIRETA.CDSERVFILI = V_CDSERVAJUS) THEN
               V_QTEFETSERV := V_QTEFETSERV + (QTDEMINIEFET - V_QTEFETGRUP);
            END IF;
         ELSE
         -- POR PRECO
            UTIL_APURAC.PRECOCORR(REC_GDIRETA.CDFILIAL,
                                  REC_GDIRETA.CDCLIENTE,
                                  REC_GDIRETA.CDSERVFILI,
                                  REC_GDIRETA.CDGRSERVFAT,
                                  V_QTEFETGRUP,
                                  QTDEMINIEFET,
                                  'G',
                                  V_VRSERVICO,
                                  V_VRPERCCORR);
         END IF;
      ELSE
      -- POR PERIODO

         /* VERIFICA SE JA FOI GRAVADO REGISTRO PARA AJUSTE DE EFETIVOS
            POR QUANTIDADE */
         SELECT COUNT(*)
         INTO V_EXISTE
         FROM TRAB_APURAC_GD
         WHERE CDFILIALAPGD   = P_CDFILIAL AND
               NRAPURACGD     = P_NRAPURACFAT AND
               CDCLIENTEAPGD  = REC_GDIRETA.CDCLIENTE AND
               CDSERVFILIAPGD = REC_GDIRETA.CDSERVFILI AND
               IDAJUSEFETAPGD = 'S';

         IF (V_EXISTE = 0) THEN
            /* CALCULA O TOTAL DE EFETIVOS DO PERIODO */
            SELECT SUM(QTEFETIVOS)
            INTO V_QTEFETGRUP
            FROM SUM_EFETIVOS
            WHERE (DTFATURAM BETWEEN P_DATAINI AND P_DATAFIM) AND
                  ((IDFATUPLAN = 'F') OR (STIDUTEFEPLA = 'S')) AND
                  CDFILIAL    = REC_GDIRETA.CDFILIAL AND
                  CDCLIENTE   = REC_GDIRETA.CDCLIENTE AND
                  CDGRSERVFAT = REC_GDIRETA.CDGRSERVFAT;

            IF (STIDQTDEPREC = '1') THEN
            -- POR QUANTIDADE
               IF (V_QTEFETGRUP < QTDEMINIEFET) AND (REC_GDIRETA.CDSERVFILI = V_CDSERVAJUS) THEN
                  SELECT MAX(VRSERVFATU)
                  INTO V_VRSERVFATU
                  FROM SERVCLIE
                  WHERE CDFILIAL   = REC_GDIRETA.CDFILIAL AND
                        CDCLIENTE  = REC_GDIRETA.CDCLIENTE AND
                        CDSERVFILI = REC_GDIRETA.CDSERVFILI;

                  IF (V_VRSERVFATU IS NULL) THEN
                     V_VRSERVFATU := 0;
                  END IF;

                  CONSULTA_CST('S', V_CDEMPRESA, REC_GDIRETA.CDPRODUTO, V_CDINSCESTA,
                               '01', P_CDFILIAL, P_NRAPURACFAT, V_CDGRUPOFISC);

                  SELECT NVL(MAX(CDSITTRIBTMP),'00')
                  INTO V_CSTTEMP
                  FROM CSTTEMP
                  WHERE NRCSTTEMP = V_CHAVECST;

                  IF SUBSTR(LPAD(V_CSTTEMP,2,'0'),2,1) = '6' THEN
                     V_IDPRODSUBS := 'S';
                  ELSE
                     V_IDPRODSUBS := 'N';
                  END IF;

                  /* GRAVA O AJUSTE DA QUANTIDADE DE EFETIVOS */
                  INSERT INTO TRAB_APURAC_GD
                  (CDFILIALAPGD, NRAPURACGD, CDCLIENTEAPGD,
                   CDSERVFILIAPGD, DTAPURACFATGD, CDPRODUTOAPGD,
                   NREFETIVOSAPGD, VRPRECUNITAPGD,
                   IDAJUSEFETAPGD)
                  VALUES (P_CDFILIAL, P_NRAPURACFAT, REC_GDIRETA.CDCLIENTE,
                          REC_GDIRETA.CDSERVFILI, P_DATAINI, REC_GDIRETA.CDPRODUTO,
                          QTDEMINIEFET - V_QTEFETGRUP, V_VRSERVFATU, 'S');

                  DELETE CSTTEMP
                  WHERE NRCSTTEMP = V_CHAVECST;
               END IF;
            ELSE
            -- POR PRECO
               IF (V_QTEFETGRUP < QTDEMINIEFET) THEN
                  UTIL_APURAC.PRECOCORR(REC_GDIRETA.CDFILIAL,
                                        REC_GDIRETA.CDCLIENTE,
                                        REC_GDIRETA.CDSERVFILI,
                                        REC_GDIRETA.CDGRSERVFAT,
                                        V_QTEFETGRUP,
                                        QTDEMINIEFET,
                                        'G',
                                        V_VRSERVICO,
                                        V_VRPERCCORR);
               END IF;
            END IF;
         END IF;
      END IF;

      /* GRAVA OS SERVICOS NA TRAB_APURAC_GD JA COM AS QUANTIDADES
         E VALORES CORRIGIDOS */
      SELECT COUNT(*)
      INTO V_EXISTE
      FROM TRAB_APURAC_GD
      WHERE CDFILIALAPGD   = P_CDFILIAL AND
            NRAPURACGD     = P_NRAPURACFAT AND
            CDCLIENTEAPGD  = REC_GDIRETA.CDCLIENTE AND
            CDSERVFILIAPGD = REC_GDIRETA.CDSERVFILI AND
            VRPRECUNITAPGD = ROUND(V_VRSERVICO,2);

      IF (V_EXISTE = 0) THEN
         CONSULTA_CST('S', V_CDEMPRESA, REC_GDIRETA.CDPRODUTO, V_CDINSCESTA,
                      '01', P_CDFILIAL, P_NRAPURACFAT, V_CDGRUPOFISC);

         SELECT NVL(MAX(CDSITTRIBTMP),'00')
         INTO V_CSTTEMP
         FROM CSTTEMP
         WHERE NRCSTTEMP = V_CHAVECST;

         IF SUBSTR(LPAD(V_CSTTEMP,2,'0'),2,1) = '6' THEN
            V_IDPRODSUBS := 'S';
         ELSE
            V_IDPRODSUBS := 'N';
         END IF;

         INSERT INTO TRAB_APURAC_GD
         (CDFILIALAPGD, NRAPURACGD, CDCLIENTEAPGD,
          CDSERVFILIAPGD, DTAPURACFATGD, CDPRODUTOAPGD,
          NREFETIVOSAPGD, VRPRECUNITAPGD,
          IDAJUSEFETAPGD)
         VALUES (P_CDFILIAL, P_NRAPURACFAT, REC_GDIRETA.CDCLIENTE,
                 REC_GDIRETA.CDSERVFILI, P_DATAINI, REC_GDIRETA.CDPRODUTO,
                 V_QTEFETSERV, ROUND(V_VRSERVICO,2), 'N');

         DELETE CSTTEMP
         WHERE NRCSTTEMP = V_CHAVECST;
      ELSE
         UPDATE TRAB_APURAC_GD
         SET NREFETIVOSAPGD = NREFETIVOSAPGD + V_QTEFETSERV
         WHERE CDFILIALAPGD   = P_CDFILIAL AND
               NRAPURACGD     = P_NRAPURACFAT AND
               CDCLIENTEAPGD  = REC_GDIRETA.CDCLIENTE AND
               CDSERVFILIAPGD = REC_GDIRETA.CDSERVFILI AND
               VRPRECUNITAPGD = ROUND(V_VRSERVICO,2);
      END IF;
   END LOOP;
   CLOSE CUR_GDIRETA;

   /* EFETUA O CADASTRAMENTO DO SERVICOS NA TABELA DE NOTA E ITENS */
   IF NVL(P_IDGERANOTA,'S') <> 'N' THEN

      SELECT NVL(MAX(NRMAXITNF),10)
      INTO V_TTITENSNF
      FROM SERIENF
      WHERE CDEMPRESA  = V_CDEMPRESA  AND
            CDINSCESTA = V_CDINSCESTA AND
            CDSERIE    = V_CDSERIE;

      SELECT MAX(IDTPIJURCLIE), MAX(NRINSJURCLIE)
      INTO V_IDTPIJURCLIE, V_NRINSJURCLIE
      FROM CLIENTE
      WHERE CDCLIENTE = P_CDCLIENTE;

      /* APAGA TODAS AS NOTAS FISCAIS APURADAS ANTERIORMENTE COM MESMO
         NUMERO DE APURACAO */
      FOR CURS_NF IN (SELECT CDFILIAL, NRLANCTONF
                      FROM NOTAFISCAL
                      WHERE (CDFILIAL    = P_CDFILIAL) AND
                            (NRAPURACFAT = P_NRAPURACFAT)) LOOP
         DELETA_NF(CURS_NF.CDFILIAL, CURS_NF.NRLANCTONF);
      END LOOP;

      IF STIDUTEFEPLA = 'S' THEN
         UPDATE EFETCLIE
         SET CDFILIFATU  = NULL,
             NRAPURACFAT = NULL
         WHERE CDFILIAL    = V_CDFILIAPUR AND
               CDCLIENTE   = P_CDCLIENTE  AND
               DTEFETCLIE >= P_DATAINI;
      END IF;

      UPDATE EFECLIFAT
      SET CDFILIFATU  = NULL,
          NRAPURACFAT = NULL
      WHERE CDFILIAL   = V_CDFILIAPUR AND
            CDCLIENTE  = P_CDCLIENTE  AND
            DTFATURAM >= P_DATAINI;

      V_IDNOVANOTA   := 'N';
      V_CONTITEM     := V_TTITENSNF;

      FOR CURS_APURAC IN (SELECT SUM(NREFETIVOSAPGD*VRPRECUNITAPGD) VRFATUREFET
                          FROM TRAB_APURAC_GD, SERVICO
                          WHERE (TRAB_APURAC_GD.CDFILIALAPGD   = P_CDFILIAL)         AND
                                (TRAB_APURAC_GD.NRAPURACGD     = P_NRAPURACFAT)      AND
                                (TRAB_APURAC_GD.CDSERVFILIAPGD = SERVICO.CDSERVFILI) AND
                                (SERVICO.CDFILIAL = V_CDFILIAPUR)) LOOP

         IF STIDUTEFEPLA = 'S' THEN
            UPDATE EFETCLIE
            SET CDFILIFATU  = P_CDFILIAL,
                NRAPURACFAT = P_NRAPURACFAT
            WHERE CDFILIAL   = V_CDFILIAPUR AND
                  CDCLIENTE  = P_CDCLIENTE  AND
                  DTEFETCLIE BETWEEN P_DATAINI AND P_DATAFIM;
         END IF;

         UPDATE EFECLIFAT
         SET CDFILIFATU  = P_CDFILIAL,
             NRAPURACFAT = P_NRAPURACFAT
         WHERE CDFILIAL   = V_CDFILIAPUR AND
               CDCLIENTE  = P_CDCLIENTE  AND
               DTFATURAM BETWEEN P_DATAINI AND P_DATAFIM;

         IF (V_CONTITEM = V_TTITENSNF) THEN
            /* EFETUA O FECHAMENTO DA NOTA, GERACAO DE IMPOSTOS */
            IF (V_IDNOVANOTA = 'S') THEN
               GERA_NOTAFISCAL_FECHAMENTO(P_CDFILIAL, V_NRLANCTONF);
            END IF;

            V_IDNOVANOTA := 'S';
            V_CONTITEM   := 0;

            NOVO_CODIGO('LANCNF/FILIAL' || P_CDFILIAL,V_NRLANCTONF);
            V_NRLANCTONF := SUBSTR(V_NRLANCTONF,11,10);

            V_NRSEQITPROD := 0;

            GERA_NOTAFISCAL_CORPO (P_CDFILIAL,    V_CDFILIAPUR, V_EMPREATAC,
                                   '01',          V_CDSERIE,    P_CDCLIENTE,
                                   P_DATAFIM,     'FAI',        V_NRLANCTONF,
                                   V_PRXLANCESTQ, 'N',          P_NRAPURACFAT,
                                   V_NRLANCTONF);

            IF (V_DESCDOTACAO = '1') AND (V_VRCREDIDEBI <> 0) THEN
               GERA_NOTAFISCAL_ENCARGO(P_CDFILIAL, V_NRLANCTONF, V_ENCARGO, V_VRCREDIDEBI);
            END IF;

         END IF;

         V_NRSEQITPROD := V_NRSEQITPROD + 1;
         V_CONTITEM := V_CONTITEM + 1;

         SELECT MAX(NMPRODUTO), MAX(CDARVPROD)
         INTO V_NMPRODUTO, V_CDARVPROD
         FROM PRODUTO
         WHERE CDPRODUTO = V_CDPRODUTO;

         GERA_NOTAFISCAL_PRODUTO(P_CDFILIAL, V_NRLANCTONF, V_CDPRODUTO, V_CDARVPROD, V_NMPRODUTO,
                                 1, ROUND((CURS_APURAC.VRFATUREFET * (V_PERCGERNFSERV/100)),3));

         V_VRTOTALFATUR    := ROUND(V_VRTOTALFATUR + CURS_APURAC.VRFATUREFET,3);
         V_VRTOTNFSERV     := ROUND(V_VRTOTNFSERV +  (CURS_APURAC.VRFATUREFET * (V_PERCGERNFSERV/100)),3);

      END LOOP;

      /* EFETUA O FECHAMENTO DA NOTA, GERACAO DE IMPOSTOS */
      IF (V_IDNOVANOTA = 'S') THEN
         GERA_NOTAFISCAL_FECHAMENTO(P_CDFILIAL, V_NRLANCTONF);

         IF (V_DESCDOTACAO = '2') AND (V_VRCREDIDEBI <> 0) THEN
            INSERT INTO AJCTRECEBER
            (CDFILIAL,    NRLANCTONF,  DTAJUSTEREC, CDAJUCTREC,
             DSAJUSTEREC, VRAJUSTEREC, NRSEQVENREC, NRPARCELAREC)
            (SELECT MAX(CDFILIAL),  MAX(NRLANCTONF), SYSDATE , V_CDAJUSTPADR,
                    V_DSAJUCTREC,   ROUND(V_VRCREDIDEBI,2), MAX(NRSEQVENREC), MAX(NRPARCELAREC)
             FROM CONTARECEBER
             WHERE (CDFILIAL    =  P_CDFILIAL)    AND
                   (NRLANCTONF  =  V_NRLANCTONF));
         END IF;

      END IF;
      V_CONTITEM   := V_TTITENSNF;

      IF (V_VRTOTALFATUR > 0) THEN

         FOR C_NFPRODPERC IN(SELECT P.CDFILIAL,   P.CDPRODUTO,
                                    (VRPERCPROD/100) AS VRPERCPROD,
                                    PR.NMPRODUTO,  VRUNITPRFATU
                             FROM PERCPRODFATU P, PRODUTO PR
                             WHERE P.CDFILIAL     = V_CDFILIAPUR   AND
                                   P.CDPRODUTO    = PR.CDPRODUTO) LOOP

            IF (V_CONTITEM = V_TTITENSNF) THEN
               NOVO_CODIGO('LANCNF/FILIAL' || P_CDFILIAL,V_NRLANCTONF);
               V_NRLANCTONF := SUBSTR(V_NRLANCTONF,11,10);

               V_NRSEQITPROD := 0;

               GERA_NOTAFISCAL_CORPO (P_CDFILIAL,    V_CDFILIAPUR, V_EMPREOPER,
                                      '01',          V_CDSERIEO,   P_CDCLIENTE,
                                      P_DATAFIM,     'FAI',        V_NRLANCTONF,
                                      V_PRXLANCESTQ, 'N',          P_NRAPURACFAT,
                                      V_NRLANCTONF);
               IF (V_DESCDOTACAO = '1') AND (V_VRCREDIDEBI <> 0) THEN
                    GERA_NOTAFISCAL_ENCARGO(P_CDFILIAL, V_NRLANCTONF, V_ENCARGO, V_VRCREDIDEBI);
               END IF;
               V_IDNOVANOTA := 'S';
            END IF;

            V_NRSEQITPROD := V_NRSEQITPROD + 1;
            V_CONTITEM := V_CONTITEM + 1;
            IF (C_NFPRODPERC.VRUNITPRFATU > 0) THEN
               V_QTITEMPROD    := ROUND((((V_VRTOTALFATUR -  V_VRTOTNFSERV)* C_NFPRODPERC.VRPERCPROD)/C_NFPRODPERC.VRUNITPRFATU),3);
               V_VRUNITPRFATU  := ROUND(C_NFPRODPERC.VRUNITPRFATU,3);
            ELSE
               V_QTITEMPROD    := 1;
               V_VRUNITPRFATU  := ROUND(((V_VRTOTALFATUR -  V_VRTOTNFSERV)* C_NFPRODPERC.VRPERCPROD),3);
            END IF;

            GERA_NOTAFISCAL_PRODUTO(P_CDFILIAL, V_NRLANCTONF, C_NFPRODPERC.CDPRODUTO,
                                    C_NFPRODPERC.CDPRODUTO, C_NFPRODPERC.NMPRODUTO,
                                    V_QTITEMPROD, V_VRUNITPRFATU);
         END LOOP;

         IF (V_IDNOVANOTA = 'S') THEN
               GERA_NOTAFISCAL_FECHAMENTO(P_CDFILIAL, V_NRLANCTONF);
         END IF;

      END IF;
      /* APAGA OS REGISTROS DA TRAB_APURAC_GD */

      DELETE FROM TRAB_APURAC_GD
      WHERE CDFILIALAPGD  = P_CDFILIAL    AND
            NRAPURACGD    = P_NRAPURACFAT AND
            CDCLIENTEAPGD = P_CDCLIENTE;

      /* ISTO CONFIRMA A ULTIMA NOTA GERADA */
      COMMIT;
   END IF;
END;
/


